@@ -14,7 +14,7 @@ from django.conf import settings
 from django.core.files import File
 from django.core.files.storage import FileSystemStorage
 from django.db import transaction
-from django.http import HttpResponse
+from django.http import FileResponse, HttpResponse
 from django.utils.decorators import method_decorator
 from drf_yasg import openapi as openapi
 from drf_yasg.utils import swagger_auto_schema
@@ -43,6 +43,9 @@ logger = logging.getLogger(__name__)
     decorator=swagger_auto_schema(
         tags=['Export'],
         operation_summary='Get export formats',
+        x_fern_sdk_group_name=['projects', 'exports'],
+        x_fern_sdk_method_name='list_formats',
+        x_fern_audiences=['public'],
         operation_description='Retrieve the available export formats for the current project by ID.',
         manual_parameters=[
             openapi.Parameter(
@@ -80,6 +83,9 @@ class ExportFormatsListAPI(generics.RetrieveAPIView):
 @method_decorator(
     name='get',
     decorator=swagger_auto_schema(
+        x_fern_sdk_group_name='projects',
+        x_fern_sdk_method_name='create_export',
+        x_fern_audiences=['public'],
         manual_parameters=[
             openapi.Parameter(
                 name='export_type',
@@ -100,7 +106,7 @@ class ExportFormatsListAPI(generics.RetrieveAPIView):
                 type=openapi.TYPE_BOOLEAN,
                 in_=openapi.IN_QUERY,
                 description="""
-                          If true, download all resource files such as images, audio, and others relevant to the tasks. 
+                          If true, download all resource files such as images, audio, and others relevant to the tasks.
                           """,
             ),
             openapi.Parameter(
@@ -122,9 +128,9 @@ class ExportFormatsListAPI(generics.RetrieveAPIView):
         tags=['Export'],
         operation_summary='Easy export of tasks and annotations',
         operation_description="""
-        <i>Note: if you have a large project it's recommended to use 
+        <i>Note: if you have a large project it's recommended to use
         export snapshots, this easy export endpoint might have timeouts.</i><br/><br>
-        Export annotated tasks as a file in a specific format. 
+        Export annotated tasks as a file in a specific format.
         For example, to export JSON annotations for a project to a file called `annotations.json`,
         run the following from the command line:
         ```bash
@@ -197,14 +203,13 @@ class ExportAPI(generics.RetrieveAPIView):
             ).data
         logger.debug('Prepare export files')
 
-        export_stream, content_type, filename = DataExport.generate_export_file(
+        export_file, content_type, filename = DataExport.generate_export_file(
             project, tasks, export_type, download_resources, request.GET
         )
 
-        response = HttpResponse(File(export_stream), content_type=content_type)
-        response['Content-Disposition'] = 'attachment; filename="%s"' % filename
-        response['filename'] = filename
-        return response
+        r = FileResponse(export_file, as_attachment=True, content_type=content_type, filename=filename)
+        r['filename'] = filename
+        return r
 
 
 @method_decorator(
@@ -265,6 +270,9 @@ class ProjectExportFilesAuthCheck(APIView):
     name='get',
     decorator=swagger_auto_schema(
         tags=['Export'],
+        x_fern_sdk_group_name=['projects', 'exports'],
+        x_fern_sdk_method_name='list',
+        x_fern_audiences=['public'],
         operation_summary='List all export snapshots',
         operation_description="""
         Returns a list of exported files for a specific project by ID.
@@ -284,6 +292,9 @@ class ProjectExportFilesAuthCheck(APIView):
     decorator=swagger_auto_schema(
         tags=['Export'],
         operation_summary='Create new export snapshot',
+        x_fern_sdk_group_name=['projects', 'exports'],
+        x_fern_sdk_method_name='create',
+        x_fern_audiences=['public'],
         operation_description="""
         Create a new export request to start a background task and generate an export file for a specific project by ID.
         """,
@@ -355,6 +366,9 @@ class ExportListAPI(generics.ListCreateAPIView):
     name='get',
     decorator=swagger_auto_schema(
         tags=['Export'],
+        x_fern_sdk_group_name=['projects', 'exports'],
+        x_fern_sdk_method_name='get',
+        x_fern_audiences=['public'],
         operation_summary='Get export snapshot by ID',
         operation_description="""
         Retrieve information about an export file by export ID for a specific project.
@@ -379,6 +393,9 @@ class ExportListAPI(generics.ListCreateAPIView):
     name='delete',
     decorator=swagger_auto_schema(
         tags=['Export'],
+        x_fern_sdk_group_name=['projects', 'exports'],
+        x_fern_sdk_method_name='delete',
+        x_fern_audiences=['public'],
         operation_summary='Delete export snapshot',
         operation_description="""
         Delete an export file by specified export ID.
@@ -443,12 +460,15 @@ class ExportDetailAPI(generics.RetrieveDestroyAPIView):
     name='get',
     decorator=swagger_auto_schema(
         tags=['Export'],
+        x_fern_sdk_group_name=['projects', 'exports'],
+        x_fern_sdk_method_name='download',
+        x_fern_audiences=['public'],
         operation_summary='Download export snapshot as file in specified format',
         operation_description="""
-        Download an export file in the specified format for a specific project. Specify the project ID with the `id` 
-        parameter in the path and the ID of the export file you want to download using the `export_pk` parameter 
-        in the path. 
-        
+        Download an export file in the specified format for a specific project. Specify the project ID with the `id`
+        parameter in the path and the ID of the export file you want to download using the `export_pk` parameter
+        in the path.
+
         Get the `export_pk` from the response of the request to [Create new export](/api#operation/api_projects_exports_create)
         or after [listing export files](/api#operation/api_projects_exports_list).
         """,
@@ -591,6 +611,9 @@ def set_convert_background_failure(job, connection, type, value, traceback_obj):
     name='post',
     decorator=swagger_auto_schema(
         tags=['Export'],
+        x_fern_sdk_group_name=['projects', 'exports'],
+        x_fern_sdk_method_name='convert',
+        x_fern_audiences=['public'],
         operation_summary='Export conversion',
         operation_description="""
         Convert export snapshot to selected format
