@@ -11,7 +11,13 @@ from data_manager.actions import get_all_actions, perform_action
 from data_manager.functions import evaluate_predictions, get_prepare_params, get_prepared_queryset
 from data_manager.managers import get_fields_for_evaluation
 from data_manager.models import View
-from data_manager.serializers import DataManagerTaskSerializer, ViewResetSerializer, ViewSerializer
+from data_manager.prepare_params import filters_schema, ordering_schema, prepare_params_schema
+from data_manager.serializers import (
+    DataManagerTaskSerializer,
+    ViewOrderSerializer,
+    ViewResetSerializer,
+    ViewSerializer,
+)
 from django.conf import settings
 from django.utils.decorators import method_decorator
 from django_filters.rest_framework import DjangoFilterBackend
@@ -28,11 +34,26 @@ from tasks.models import Annotation, Prediction, Task
 
 logger = logging.getLogger(__name__)
 
+_view_request_body = openapi.Schema(
+    type=openapi.TYPE_OBJECT,
+    properties={
+        'data': openapi.Schema(
+            type=openapi.TYPE_OBJECT,
+            description='Custom view data',
+            properties={'filters': filters_schema, 'ordering': ordering_schema},
+        ),
+        'project': openapi.Schema(type=openapi.TYPE_INTEGER, description='Project ID'),
+    },
+)
+
 
 @method_decorator(
     name='list',
     decorator=swagger_auto_schema(
         tags=['Data Manager'],
+        x_fern_sdk_group_name='views',
+        x_fern_sdk_method_name='list',
+        x_fern_audiences=['public'],
         operation_summary='List views',
         operation_description='List all views for a specific project.',
         manual_parameters=[
@@ -46,14 +67,22 @@ logger = logging.getLogger(__name__)
     name='create',
     decorator=swagger_auto_schema(
         tags=['Data Manager'],
+        x_fern_sdk_group_name='views',
+        x_fern_sdk_method_name='create',
+        x_fern_audiences=['public'],
         operation_summary='Create view',
         operation_description='Create a view for a specific project.',
+        request_body=_view_request_body,
+        responses={201: ViewSerializer},
     ),
 )
 @method_decorator(
     name='retrieve',
     decorator=swagger_auto_schema(
         tags=['Data Manager'],
+        x_fern_sdk_group_name='views',
+        x_fern_sdk_method_name='get',
+        x_fern_audiences=['public'],
         operation_summary='Get view details',
         operation_description='Get the details about a specific view in the data manager',
         manual_parameters=[
@@ -65,8 +94,10 @@ logger = logging.getLogger(__name__)
     name='update',
     decorator=swagger_auto_schema(
         tags=['Data Manager'],
+        x_fern_audiences=['internal'],
         operation_summary='Put view',
         operation_description='Overwrite view data with updated filters and other information for a specific project.',
+        request_body=_view_request_body,
         manual_parameters=[
             openapi.Parameter(name='id', type=openapi.TYPE_STRING, in_=openapi.IN_PATH, description='View ID'),
         ],
@@ -76,17 +107,25 @@ logger = logging.getLogger(__name__)
     name='partial_update',
     decorator=swagger_auto_schema(
         tags=['Data Manager'],
+        x_fern_sdk_group_name='views',
+        x_fern_sdk_method_name='update',
+        x_fern_audiences=['public'],
         operation_summary='Update view',
         operation_description='Update view data with additional filters and other information for a specific project.',
         manual_parameters=[
             openapi.Parameter(name='id', type=openapi.TYPE_STRING, in_=openapi.IN_PATH, description='View ID'),
         ],
+        request_body=_view_request_body,
+        responses={200: ViewSerializer},
     ),
 )
 @method_decorator(
     name='destroy',
     decorator=swagger_auto_schema(
         tags=['Data Manager'],
+        x_fern_sdk_group_name='views',
+        x_fern_sdk_method_name='delete',
+        x_fern_audiences=['public'],
         operation_summary='Delete view',
         operation_description='Delete a specific view by ID.',
         manual_parameters=[
@@ -111,8 +150,11 @@ class ViewAPI(viewsets.ModelViewSet):
 
     @swagger_auto_schema(
         tags=['Data Manager'],
-        operation_summary='Reset project views',
-        operation_description='Reset all views for a specific project.',
+        x_fern_sdk_group_name='views',
+        x_fern_sdk_method_name='delete_all',
+        x_fern_audiences=['public'],
+        operation_summary='Delete all project views',
+        operation_description='Delete all views for a specific project',
         request_body=ViewResetSerializer,
     )
     @action(detail=False, methods=['delete'])
@@ -126,8 +168,39 @@ class ViewAPI(viewsets.ModelViewSet):
         queryset.all().delete()
         return Response(status=204)
 
+    @swagger_auto_schema(
+        method='post',
+        tags=['Data Manager'],
+        operation_summary='Update order of views',
+        operation_description='Update the order field of views based on the provided list of view IDs',
+        request_body=ViewOrderSerializer,
+    )
+    @action(detail=False, methods=['post'], url_path='order')
+    def update_order(self, request):
+        serializer = ViewOrderSerializer(data=request.data)
+        serializer.is_valid(raise_exception=True)
+
+        project_id = serializer.validated_data['project']
+        view_ids = serializer.validated_data['ids']
+
+        project = generics.get_object_or_404(Project.objects.for_user(request.user), pk=project_id)
+
+        queryset = self.filter_queryset(self.get_queryset()).filter(project=project)
+        views = list(queryset.filter(id__in=view_ids))
+
+        # Update the order field for each view
+        view_dict = {view.id: view for view in views}
+        for order, view_id in enumerate(view_ids):
+            if view_id in view_dict:
+                view_dict[view_id].order = order
+
+        # Bulk update views
+        View.objects.bulk_update(views, ['order'])
+
+        return Response(status=200)
+
     def get_queryset(self):
-        return View.objects.filter(project__organization=self.request.user.active_organization).order_by('id')
+        return View.objects.filter(project__organization=self.request.user.active_organization).order_by('order', 'id')
 
 
 class TaskPagination(PageNumberPagination):
@@ -177,6 +250,7 @@ class TaskListAPI(generics.ListCreateAPIView):
         PUT=all_permissions.tasks_change,
         DELETE=all_permissions.tasks_delete,
     )
+    pagination_class = TaskPagination
 
     @staticmethod
     def get_task_serializer_context(request, project):
@@ -228,7 +302,6 @@ class TaskListAPI(generics.ListCreateAPIView):
         context = self.get_task_serializer_context(self.request, project)
 
         # paginated tasks
-        self.pagination_class = TaskPagination
         page = self.paginate_queryset(queryset)
 
         # get request params
@@ -257,6 +330,10 @@ class TaskListAPI(generics.ListCreateAPIView):
 
             # retrieve ML predictions if tasks don't have them
             if not review and project.evaluate_predictions_automatically:
+                # TODO MM TODO this needs a discussion, because I'd expect
+                # people to retrieve manually instead on DM load, plus it
+                # will slow down initial DM load
+                # if project.retrieve_predictions_automatically is deprecated now and no longer used
                 tasks_for_predictions = Task.objects.filter(id__in=ids, predictions__isnull=True)
                 evaluate_predictions(tasks_for_predictions)
                 [tasks_by_ids[_id].refresh_from_db() for _id in ids]
@@ -285,8 +362,53 @@ class TaskListAPI(generics.ListCreateAPIView):
     name='get',
     decorator=swagger_auto_schema(
         tags=['Data Manager'],
+        x_fern_audiences=['internal'],
         operation_summary='Get data manager columns',
-        operation_description='Retrieve the data manager columns available for the tasks in a specific project.',
+        operation_description=(
+            'Retrieve the data manager columns available for the tasks in a specific project. '
+            'For more details, see [GET api/actions](#/Data%20Manager/get_api_actions).'
+        ),
+        manual_parameters=[
+            openapi.Parameter(
+                name='project',
+                type=openapi.TYPE_INTEGER,
+                in_=openapi.IN_QUERY,
+                description='Project ID',
+                required=True,
+            )
+        ],
+        responses={
+            200: openapi.Response(
+                description='Columns retrieved successfully',
+                examples={
+                    'application/json': {
+                        'columns': [
+                            {
+                                'id': 'id',
+                                'title': 'ID',
+                                'type': 'Number',
+                                'help': 'Task ID',
+                                'target': 'tasks',
+                                'visibility_defaults': {'explore': True, 'labeling': False},
+                                'project_defined': False,
+                            },
+                            {
+                                'id': 'completed_at',
+                                'title': 'Completed',
+                                'type': 'Datetime',
+                                'target': 'tasks',
+                                'help': 'Last annotation date',
+                                'visibility_defaults': {'explore': True, 'labeling': False},
+                                'project_defined': False,
+                            },
+                            # ... other columns ...
+                        ]
+                    }
+                },
+            ),
+            400: openapi.Response(description='Invalid project ID supplied'),
+            404: openapi.Response(description='Project not found'),
+        },
     ),
 )
 class ProjectColumnsAPI(APIView):
@@ -305,6 +427,7 @@ class ProjectColumnsAPI(APIView):
     name='get',
     decorator=swagger_auto_schema(
         tags=['Data Manager'],
+        x_fern_audiences=['internal'],
         operation_summary='Get project state',
         operation_description='Retrieve the project state for the data manager.',
     ),
@@ -337,6 +460,9 @@ class ProjectStateAPI(APIView):
     name='get',
     decorator=swagger_auto_schema(
         tags=['Data Manager'],
+        x_fern_sdk_group_name='actions',
+        x_fern_sdk_method_name='list',
+        x_fern_audiences=['public'],
         operation_summary='Get actions',
         operation_description='Retrieve all the registered actions with descriptions that data manager can use.',
     ),
@@ -345,8 +471,54 @@ class ProjectStateAPI(APIView):
     name='post',
     decorator=swagger_auto_schema(
         tags=['Data Manager'],
+        x_fern_sdk_group_name='actions',
+        x_fern_sdk_method_name='create',
+        x_fern_audiences=['public'],
         operation_summary='Post actions',
-        operation_description='Perform an action with the selected items from a specific view.',
+        operation_description=(
+            'Perform a Data Manager action with the selected tasks and filters. '
+            'Note: More complex actions require additional parameters in the request body. '
+            'Call `GET api/actions?project=<id>` to explore them. <br>'
+            'Example: `GET api/actions?id=delete_tasks&project=1`'
+        ),
+        request_body=prepare_params_schema,
+        manual_parameters=[
+            openapi.Parameter(
+                name='id',
+                type=openapi.TYPE_STRING,
+                in_=openapi.IN_QUERY,
+                description='Action name ID, see the full list of actions in the `GET api/actions` request',
+                enum=[
+                    'retrieve_tasks_predictions',
+                    'predictions_to_annotations',
+                    'remove_duplicates',
+                    'delete_tasks',
+                    'delete_ground_truths',
+                    'delete_tasks_annotations',
+                    'delete_tasks_reviews',
+                    'delete_tasks_predictions',
+                    'delete_reviewers',
+                    'delete_annotators',
+                ],
+                example='delete_tasks',
+                required=True,
+            ),
+            openapi.Parameter(
+                name='project',
+                type=openapi.TYPE_INTEGER,
+                in_=openapi.IN_QUERY,
+                description='Project ID',
+                required=True,
+            ),
+            openapi.Parameter(
+                name='view',
+                type=openapi.TYPE_INTEGER,
+                in_=openapi.IN_QUERY,
+                description='View ID (optional, it has higher priority than filters, '
+                'selectedItems and ordering from the request body payload)',
+            ),
+        ],
+        responses={200: openapi.Response(description='Action performed successfully')},
     ),
 )
 class ProjectActionsAPI(APIView):
