@@ -3,16 +3,32 @@
 import logging
 import re
 from datetime import datetime
+from typing import ClassVar
 
 import ujson as json
 from core.feature_flags import flag_set
+from core.utils.db import fast_first
 from data_manager.prepare_params import ConjunctionEnum
 from django.conf import settings
 from django.contrib.postgres.aggregates import ArrayAgg
 from django.contrib.postgres.fields.jsonb import KeyTextTransform
 from django.db import models
-from django.db.models import Aggregate, Avg, Case, Exists, F, FloatField, OuterRef, Q, Subquery, Value, When
-from django.db.models.functions import Cast, Coalesce
+from django.db.models import (
+    Aggregate,
+    Avg,
+    Case,
+    DateTimeField,
+    Exists,
+    F,
+    FloatField,
+    OuterRef,
+    Q,
+    Subquery,
+    TextField,
+    Value,
+    When,
+)
+from django.db.models.functions import Cast, Coalesce, Concat
 from pydantic import BaseModel
 
 from label_studio.core.utils.common import load_func
@@ -24,20 +40,20 @@ DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'
 
 
 class _Operator(BaseModel):
-    EQUAL = 'equal'
-    NOT_EQUAL = 'not_equal'
-    LESS = 'less'
-    GREATER = 'greater'
-    LESS_OR_EQUAL = 'less_or_equal'
-    GREATER_OR_EQUAL = 'greater_or_equal'
-    IN = 'in'
-    NOT_IN = 'not_in'
-    IN_LIST = 'in_list'
-    NOT_IN_LIST = 'not_in_list'
-    EMPTY = 'empty'
-    CONTAINS = 'contains'
-    NOT_CONTAINS = 'not_contains'
-    REGEX = 'regex'
+    EQUAL: ClassVar[str] = 'equal'
+    NOT_EQUAL: ClassVar[str] = 'not_equal'
+    LESS: ClassVar[str] = 'less'
+    GREATER: ClassVar[str] = 'greater'
+    LESS_OR_EQUAL: ClassVar[str] = 'less_or_equal'
+    GREATER_OR_EQUAL: ClassVar[str] = 'greater_or_equal'
+    IN: ClassVar[str] = 'in'
+    NOT_IN: ClassVar[str] = 'not_in'
+    IN_LIST: ClassVar[str] = 'in_list'
+    NOT_IN_LIST: ClassVar[str] = 'not_in_list'
+    EMPTY: ClassVar[str] = 'empty'
+    CONTAINS: ClassVar[str] = 'contains'
+    NOT_CONTAINS: ClassVar[str] = 'not_contains'
+    REGEX: ClassVar[str] = 'regex'
 
 
 Operator = _Operator()
@@ -495,11 +511,86 @@ class GroupConcat(Aggregate):
         super().__init__(expression, distinct='DISTINCT ' if distinct else '', output_field=output_field, **extra)
 
 
-def annotate_completed_at(queryset):
+def newest_annotation_subquery() -> Subquery:
     from tasks.models import Annotation
 
-    newest = Annotation.objects.filter(task=OuterRef('pk')).order_by('-id')[:1]
-    return queryset.annotate(completed_at=Case(When(is_labeled=True, then=Subquery(newest.values('created_at')))))
+    newest_annotations = Annotation.objects.filter(task=OuterRef('pk')).order_by('-id')[:1]
+    return Subquery(newest_annotations.values('created_at'))
+
+
+def base_annotate_completed_at(queryset: TaskQuerySet) -> TaskQuerySet:
+    return queryset.annotate(completed_at=Case(When(is_labeled=True, then=newest_annotation_subquery())))
+
+
+def annotate_completed_at(queryset: TaskQuerySet) -> TaskQuerySet:
+    LseProject = load_func(settings.LSE_PROJECT)
+    get_tasks_agreement_queryset = load_func(settings.GET_TASKS_AGREEMENT_QUERYSET)
+
+    is_lse_project = bool(LseProject)
+    has_custom_agreement_queryset = bool(get_tasks_agreement_queryset)
+
+    if (
+        is_lse_project
+        and has_custom_agreement_queryset
+        and flag_set('fflag_feat_optic_161_project_settings_for_low_agreement_threshold_score_short', user='auto')
+    ):
+        return annotated_completed_at_considering_agreement_threshold(queryset)
+
+    return base_annotate_completed_at(queryset)
+
+
+def annotated_completed_at_considering_agreement_threshold(queryset):
+    LseProject = load_func(settings.LSE_PROJECT)
+    get_tasks_agreement_queryset = load_func(settings.GET_TASKS_AGREEMENT_QUERYSET)
+
+    is_lse_project = bool(LseProject)
+    has_custom_agreement_queryset = bool(get_tasks_agreement_queryset)
+
+    project_exists = is_lse_project and hasattr(queryset, 'project') and queryset.project is not None
+
+    project_id = queryset.project.id if project_exists else None
+
+    if project_id is None or not is_lse_project or not has_custom_agreement_queryset:
+        return base_annotate_completed_at(queryset)
+
+    lse_project = fast_first(
+        LseProject.objects.filter(project_id=project_id).values(
+            'agreement_threshold', 'max_additional_annotators_assignable'
+        )
+    )
+
+    agreement_threshold = lse_project['agreement_threshold'] if lse_project else None
+    if not lse_project or not agreement_threshold:
+        # This project doesn't use task_agreement so don't consider it when determining completed_at
+        return base_annotate_completed_at(queryset)
+
+    queryset = get_tasks_agreement_queryset(queryset)
+    max_additional_annotators_assignable = lse_project['max_additional_annotators_assignable']
+
+    completed_at_case = Case(
+        When(
+            # If agreement_threshold is set, evaluate all conditions
+            Q(is_labeled=True)
+            & (
+                Q(_agreement__gte=agreement_threshold)
+                | Q(annotation_count__gte=(F('overlap') + max_additional_annotators_assignable))
+            ),
+            then=newest_annotation_subquery(),
+        ),
+        default=Value(None),
+        output_field=DateTimeField(),
+    )
+
+    return queryset.annotate(completed_at=completed_at_case)
+
+
+def annotate_storage_filename(queryset: TaskQuerySet) -> TaskQuerySet:
+    from label_studio.data_manager.functions import intersperse
+
+    storage_key_names = [F(s + '__key') for s in settings.IO_STORAGES_IMPORT_LINK_NAMES]
+    return queryset.annotate(
+        storage_filename=Concat(*intersperse(storage_key_names, Value(';')), output_field=TextField())
+    )
 
 
 def annotate_annotations_results(queryset):
@@ -605,6 +696,7 @@ settings.DATA_MANAGER_ANNOTATIONS_MAP = {
     'annotations_ids': annotate_annotations_ids,
     'file_upload': file_upload,
     'draft_exists': annotate_draft_exists,
+    'storage_filename': annotate_storage_filename,
 }
 
 
