@@ -9,6 +9,7 @@ from core.filters import ListFilter
 from core.label_config import config_essential_data_has_changed
 from core.mixins import GetParentObjectMixin
 from core.permissions import ViewClassPermission, all_permissions
+from core.redis import start_job_async_or_sync
 from core.utils.common import paginator, paginator_help, temporary_disconnect_all_signals
 from core.utils.exceptions import LabelStudioDatabaseException, ProjectExistException
 from core.utils.io import find_dir, find_file, read_yaml
@@ -21,13 +22,16 @@ from django.utils.decorators import method_decorator
 from django_filters import CharFilter, FilterSet
 from django_filters.rest_framework import DjangoFilterBackend
 from drf_yasg.utils import swagger_auto_schema
+from ml.serializers import MLBackendSerializer
 from projects.functions.next_task import get_next_task
 from projects.functions.stream_history import get_label_stream_history
+from projects.functions.utils import recalculate_created_annotations_and_labels_from_scratch
 from projects.models import Project, ProjectImport, ProjectManager, ProjectReimport, ProjectSummary
 from projects.serializers import (
     GetFieldsSerializer,
     ProjectImportSerializer,
     ProjectLabelConfigSerializer,
+    ProjectModelVersionExtendedSerializer,
     ProjectReimportSerializer,
     ProjectSerializer,
     ProjectSummarySerializer,
@@ -39,6 +43,7 @@ from rest_framework.pagination import PageNumberPagination
 from rest_framework.parsers import FormParser, JSONParser, MultiPartParser
 from rest_framework.permissions import AllowAny
 from rest_framework.response import Response
+from rest_framework.settings import api_settings
 from rest_framework.views import exception_handler
 from tasks.models import Task
 from tasks.serializers import (
@@ -50,8 +55,11 @@ from tasks.serializers import (
 from webhooks.models import WebhookAction
 from webhooks.utils import api_webhook, api_webhook_for_delete, emit_webhooks_for_instance
 
+from label_studio.core.utils.common import load_func
+
 logger = logging.getLogger(__name__)
 
+ProjectImportPermission = load_func(settings.PROJECT_IMPORT_PERMISSION)
 
 _result_schema = openapi.Schema(
     title='Labeling result',
@@ -84,6 +92,91 @@ _task_data_schema = openapi.Schema(
     example={'id': 1, 'my_image_url': '/static/samples/kittens.jpg'},
 )
 
+_project_schema = openapi.Schema(
+    title='Project',
+    description='Project',
+    type=openapi.TYPE_OBJECT,
+    properties={
+        'title': openapi.Schema(
+            title='title',
+            description='Project title',
+            type=openapi.TYPE_STRING,
+            example='My project',
+        ),
+        'description': openapi.Schema(
+            title='description',
+            description='Project description',
+            type=openapi.TYPE_STRING,
+            example='My first project',
+        ),
+        'label_config': openapi.Schema(
+            title='label_config',
+            description='Label config in XML format',
+            type=openapi.TYPE_STRING,
+            example='<View>[...]</View>',
+        ),
+        'expert_instruction': openapi.Schema(
+            title='expert_instruction',
+            description='Labeling instructions to show to the user',
+            type=openapi.TYPE_STRING,
+            example='Label all cats',
+        ),
+        'show_instruction': openapi.Schema(
+            title='show_instruction',
+            description='Show labeling instructions',
+            type=openapi.TYPE_BOOLEAN,
+        ),
+        'show_skip_button': openapi.Schema(
+            title='show_skip_button',
+            description='Show skip button',
+            type=openapi.TYPE_BOOLEAN,
+        ),
+        'enable_empty_annotation': openapi.Schema(
+            title='enable_empty_annotation',
+            description='Allow empty annotations',
+            type=openapi.TYPE_BOOLEAN,
+        ),
+        'show_annotation_history': openapi.Schema(
+            title='show_annotation_history',
+            description='Show annotation history',
+            type=openapi.TYPE_BOOLEAN,
+        ),
+        'reveal_preannotations_interactively': openapi.Schema(
+            title='reveal_preannotations_interactively',
+            description='Reveal preannotations interactively. If set to True, predictions will be shown to the user only after selecting the area of interest',
+            type=openapi.TYPE_BOOLEAN,
+        ),
+        'show_collab_predictions': openapi.Schema(
+            title='show_collab_predictions',
+            description='Show predictions to annotators',
+            type=openapi.TYPE_BOOLEAN,
+        ),
+        'maximum_annotations': openapi.Schema(
+            title='maximum_annotations',
+            description='Maximum annotations per task',
+            type=openapi.TYPE_INTEGER,
+        ),
+        'color': openapi.Schema(
+            title='color',
+            description='Project color in HEX format',
+            type=openapi.TYPE_STRING,
+            default='#FFFFFF',
+        ),
+        'control_weights': openapi.Schema(
+            title='control_weights',
+            description='Dict of weights for each control tag in metric calculation. Each control tag (e.g. label or choice) will '
+            'have its own key in control weight dict with weight for each label and overall weight. '
+            'For example, if a bounding box annotation with a control tag named my_bbox should be included with 0.33 weight in agreement calculation, '
+            'and the first label Car should be twice as important as Airplane, then you need to specify: '
+            "{'my_bbox': {'type': 'RectangleLabels', 'labels': {'Car': 1.0, 'Airplane': 0.5}, 'overall': 0.33}",
+            type=openapi.TYPE_OBJECT,
+            example={
+                'my_bbox': {'type': 'RectangleLabels', 'labels': {'Car': 1.0, 'Airplaine': 0.5}, 'overall': 0.33}
+            },
+        ),
+    },
+)
+
 
 class ProjectListPagination(PageNumberPagination):
     page_size = 30
@@ -99,6 +192,13 @@ class ProjectFilterSet(FilterSet):
     name='get',
     decorator=swagger_auto_schema(
         tags=['Projects'],
+        x_fern_sdk_group_name='projects',
+        x_fern_sdk_method_name='list',
+        x_fern_audiences=['public'],
+        x_fern_pagination={
+            'offset': '$request.page',
+            'results': '$response.results',
+        },
         operation_summary='List your projects',
         operation_description="""
     Return a list of the projects that you've created.
@@ -119,16 +219,20 @@ class ProjectFilterSet(FilterSet):
     decorator=swagger_auto_schema(
         tags=['Projects'],
         operation_summary='Create new project',
+        x_fern_sdk_group_name='projects',
+        x_fern_sdk_method_name='create',
+        x_fern_audiences=['public'],
         operation_description="""
     Create a project and set up the labeling interface in Label Studio using the API.
-    
+
     ```bash
     curl -H Content-Type:application/json -H 'Authorization: Token abc123' -X POST '{}/api/projects' \
-    --data '{{"label_config": "<View>[...]</View>"}}'
+    --data '{{"title": "My project", "label_config": "<View></View>"}}'
     ```
     """.format(
             settings.HOSTNAME or 'https://localhost:8080'
         ),
+        request_body=_project_schema,
     ),
 )
 class ProjectListAPI(generics.ListCreateAPIView):
@@ -181,14 +285,78 @@ class ProjectListAPI(generics.ListCreateAPIView):
     name='get',
     decorator=swagger_auto_schema(
         tags=['Projects'],
+        x_fern_sdk_group_name='projects',
+        x_fern_sdk_method_name='get',
+        x_fern_audiences=['public'],
         operation_summary='Get project by ID',
         operation_description='Retrieve information about a project by project ID.',
+        responses={
+            '200': openapi.Response(
+                description='Project information',
+                schema=ProjectSerializer,
+                examples={
+                    'application/json': {
+                        'id': 1,
+                        'title': 'My project',
+                        'description': 'My first project',
+                        'label_config': '<View>[...]</View>',
+                        'expert_instruction': 'Label all cats',
+                        'show_instruction': True,
+                        'show_skip_button': True,
+                        'enable_empty_annotation': True,
+                        'show_annotation_history': True,
+                        'organization': 1,
+                        'color': '#FF0000',
+                        'maximum_annotations': 1,
+                        'is_published': True,
+                        'model_version': '1.0.0',
+                        'is_draft': False,
+                        'created_by': {
+                            'id': 1,
+                            'first_name': 'Jo',
+                            'last_name': 'Doe',
+                            'email': 'manager@humansignal.com',
+                        },
+                        'created_at': '2023-08-24T14:15:22Z',
+                        'min_annotations_to_start_training': 0,
+                        'start_training_on_annotation_update': True,
+                        'show_collab_predictions': True,
+                        'num_tasks_with_annotations': 10,
+                        'task_number': 100,
+                        'useful_annotation_number': 10,
+                        'ground_truth_number': 5,
+                        'skipped_annotations_number': 0,
+                        'total_annotations_number': 10,
+                        'total_predictions_number': 0,
+                        'sampling': 'Sequential sampling',
+                        'show_ground_truth_first': True,
+                        'show_overlap_first': True,
+                        'overlap_cohort_percentage': 100,
+                        'task_data_login': 'user',
+                        'task_data_password': 'secret',
+                        'control_weights': {},
+                        'parsed_label_config': '{"tag": {...}}',
+                        'evaluate_predictions_automatically': False,
+                        'config_has_control_tags': True,
+                        'skip_queue': 'REQUEUE_FOR_ME',
+                        'reveal_preannotations_interactively': True,
+                        'pinned_at': '2023-08-24T14:15:22Z',
+                        'finished_task_number': 10,
+                        'queue_total': 10,
+                        'queue_done': 100,
+                    }
+                },
+            )
+        },
     ),
 )
 @method_decorator(
     name='delete',
     decorator=swagger_auto_schema(
         tags=['Projects'],
+        x_fern_sdk_group_name='projects',
+        x_fern_sdk_method_name='delete',
+        x_fern_audiences=['public'],
         operation_summary='Delete project',
         operation_description='Delete a project by specified project ID.',
     ),
@@ -197,13 +365,15 @@ class ProjectListAPI(generics.ListCreateAPIView):
     name='patch',
     decorator=swagger_auto_schema(
         tags=['Projects'],
+        x_fern_sdk_group_name='projects',
+        x_fern_sdk_method_name='update',
+        x_fern_audiences=['public'],
         operation_summary='Update project',
         operation_description='Update the project settings for a specific project.',
-        request_body=ProjectSerializer,
+        request_body=_project_schema,
     ),
 )
 class ProjectAPI(generics.RetrieveUpdateDestroyAPIView):
-
     parser_classes = (JSONParser, FormParser, MultiPartParser)
     queryset = Project.objects.with_counts()
     permission_required = ViewClassPermission(
@@ -261,6 +431,9 @@ class ProjectAPI(generics.RetrieveUpdateDestroyAPIView):
     decorator=swagger_auto_schema(
         tags=['Projects'],
         operation_summary='Get next task to label',
+        x_fern_sdk_group_name='projects',
+        x_fern_sdk_method_name='next_task',
+        x_fern_audiences=['public'],
         operation_description="""
     Get the next task for labeling. If you enable Machine Learning in
     your project, the response might include a "predictions"
@@ -271,11 +444,10 @@ class ProjectAPI(generics.RetrieveUpdateDestroyAPIView):
     ),
 )  # leaving this method decorator info in case we put it back in swagger API docs
 class ProjectNextTaskAPI(generics.RetrieveAPIView):
-
     permission_required = all_permissions.tasks_view
     serializer_class = TaskWithAnnotationsAndPredictionsAndDraftsSerializer  # using it for swagger API docs
     queryset = Project.objects.all()
-    swagger_schema = None   # this endpoint doesn't need to be in swagger API docs
+    swagger_schema = None  # this endpoint doesn't need to be in swagger API docs
 
     def get(self, request, *args, **kwargs):
         project = self.get_object()
@@ -316,6 +488,7 @@ class LabelStreamHistoryAPI(generics.RetrieveAPIView):
     name='post',
     decorator=swagger_auto_schema(
         tags=['Projects'],
+        x_fern_audiences=['internal'],
         operation_summary='Validate label config',
         operation_description='Validate an arbitrary labeling configuration.',
         responses={204: 'Validation success'},
@@ -347,7 +520,11 @@ class LabelConfigValidateAPI(generics.CreateAPIView):
     name='post',
     decorator=swagger_auto_schema(
         tags=['Projects'],
+        operation_id='api_projects_validate_label_config',
         operation_summary='Validate project label config',
+        x_fern_sdk_group_name='projects',
+        x_fern_sdk_method_name='validate_config',
+        x_fern_audiences=['public'],
         operation_description="""
         Determine whether the label configuration for a specific project is valid.
         """,
@@ -397,10 +574,39 @@ class ProjectSummaryAPI(generics.RetrieveAPIView):
         return super(ProjectSummaryAPI, self).get(*args, **kwargs)
 
 
+class ProjectSummaryResetAPI(GetParentObjectMixin, generics.CreateAPIView):
+    """This API is useful when we need to reset project.summary.created_labels and created_labels_drafts
+    and recalculate them from scratch. It's hard to correctly follow all changes in annotation region
+    labels and these fields aren't calculated properly after some time. Label config changes are not allowed
+    when these changes touch any labels from these created_labels* dictionaries.
+    """
+
+    parser_classes = (JSONParser,)
+    parent_queryset = Project.objects.all()
+    permission_required = ViewClassPermission(
+        POST=all_permissions.projects_change,
+    )
+
+    @swagger_auto_schema(auto_schema=None)
+    def post(self, *args, **kwargs):
+        project = self.get_parent_object()
+        summary = project.summary
+        start_job_async_or_sync(
+            recalculate_created_annotations_and_labels_from_scratch,
+            project,
+            summary,
+            organization_id=self.request.user.active_organization.id,
+        )
+        return Response(status=status.HTTP_200_OK)
+
+
 @method_decorator(
     name='get',
     decorator=swagger_auto_schema(
         tags=['Projects'],
+        x_fern_sdk_group_name='tasks',
+        x_fern_sdk_method_name='create_many_status',
+        x_fern_audiences=['public'],
         operation_summary='Get project import info',
         operation_description='Return data related to async project import operation',
         manual_parameters=[
@@ -414,9 +620,10 @@ class ProjectSummaryAPI(generics.RetrieveAPIView):
     ),
 )
 class ProjectImportAPI(generics.RetrieveAPIView):
+    permission_required = all_permissions.projects_change
+    permission_classes = api_settings.DEFAULT_PERMISSION_CLASSES + [ProjectImportPermission]
     parser_classes = (JSONParser,)
     serializer_class = ProjectImportSerializer
-    permission_required = all_permissions.projects_change
     queryset = ProjectImport.objects.all()
     lookup_url_kwarg = 'import_pk'
 
@@ -425,6 +632,7 @@ class ProjectImportAPI(generics.RetrieveAPIView):
     name='get',
     decorator=swagger_auto_schema(
         tags=['Projects'],
+        x_fern_audiences=['internal'],
         operation_summary='Get project reimport info',
         operation_description='Return data related to async project reimport operation',
         manual_parameters=[
@@ -438,9 +646,10 @@ class ProjectImportAPI(generics.RetrieveAPIView):
     ),
 )
 class ProjectReimportAPI(generics.RetrieveAPIView):
+    permission_required = all_permissions.projects_change
+    permission_classes = api_settings.DEFAULT_PERMISSION_CLASSES + [ProjectImportPermission]
     parser_classes = (JSONParser,)
     serializer_class = ProjectReimportSerializer
-    permission_required = all_permissions.projects_change
     queryset = ProjectReimport.objects.all()
     lookup_url_kwarg = 'reimport_pk'
 
@@ -449,6 +658,9 @@ class ProjectReimportAPI(generics.RetrieveAPIView):
     name='delete',
     decorator=swagger_auto_schema(
         tags=['Projects'],
+        x_fern_sdk_group_name='projects',
+        x_fern_sdk_method_name='delete_all_tasks',
+        x_fern_audiences=['public'],
         operation_summary='Delete all tasks',
         operation_description='Delete all tasks from a specific project.',
         manual_parameters=[
@@ -459,12 +671,14 @@ class ProjectReimportAPI(generics.RetrieveAPIView):
                 description='A unique integer value identifying this project.',
             ),
         ],
+        responses={204: 'Tasks deleted'},
     ),
 )
 @method_decorator(
     name='get',
     decorator=swagger_auto_schema(
         tags=['Projects'],
+        x_fern_audiences=['internal'],  # TODO: deprecate this endpoint in favor of tasks:tasks-list
         operation_summary='List project tasks',
         operation_description="""
             Retrieve a paginated list of tasks for a specific project. For example, use the following cURL command:
@@ -486,7 +700,6 @@ class ProjectReimportAPI(generics.RetrieveAPIView):
     ),
 )
 class ProjectTaskListAPI(GetParentObjectMixin, generics.ListCreateAPIView, generics.DestroyAPIView):
-
     parser_classes = (JSONParser, FormParser)
     queryset = Task.objects.all()
     parent_queryset = Project.objects.all()
@@ -521,7 +734,7 @@ class ProjectTaskListAPI(GetParentObjectMixin, generics.ListCreateAPIView, gener
         Task.delete_tasks_without_signals(Task.objects.filter(project=project))
         project.summary.reset()
         emit_webhooks_for_instance(request.user.active_organization, None, WebhookAction.TASKS_DELETED, task_ids)
-        return Response(data={'tasks': task_ids}, status=204)
+        return Response(status=204)
 
     def get(self, *args, **kwargs):
         return super(ProjectTaskListAPI, self).get(*args, **kwargs)
@@ -591,5 +804,33 @@ class ProjectModelVersions(generics.RetrieveAPIView):
     queryset = Project.objects.all()
 
     def get(self, request, *args, **kwargs):
+        # TODO make sure "extended" is the right word and is
+        # consistent with other APIs we've got
+        extended = self.request.query_params.get('extended', False)
+        include_live_models = self.request.query_params.get('include_live_models', False)
         project = self.get_object()
-        return Response(data=project.get_model_versions(with_counters=True))
+        data = project.get_model_versions(with_counters=True, extended=extended)
+
+        if extended:
+            serializer_models = None
+            serializer = ProjectModelVersionExtendedSerializer(data, many=True)
+
+            if include_live_models:
+                ml_models = project.get_ml_backends()
+                serializer_models = MLBackendSerializer(ml_models, many=True)
+
+            # serializer.is_valid(raise_exception=True)
+            return Response({'static': serializer.data, 'live': serializer_models and serializer_models.data})
+        else:
+            return Response(data=data)
+
+    def delete(self, request, *args, **kwargs):
+        project = self.get_object()
+        model_version = request.data.get('model_version', None)
+
+        if not model_version:
+            raise RestValidationError('model_version param is required')
+
+        count = project.delete_predictions(model_version=model_version)
+
+        return Response(data=count)
