@@ -17,7 +17,7 @@ from django.db.models import Q
 from django.utils import timezone
 from django.utils.decorators import method_decorator
 from django_filters.rest_framework import DjangoFilterBackend
-from drf_yasg.utils import swagger_auto_schema
+from drf_yasg.utils import no_body, swagger_auto_schema
 from projects.functions.stream_history import fill_history_annotation
 from projects.models import Project
 from rest_framework import generics, viewsets
@@ -25,6 +25,15 @@ from rest_framework.exceptions import PermissionDenied
 from rest_framework.parsers import FormParser, JSONParser, MultiPartParser
 from rest_framework.response import Response
 from tasks.models import Annotation, AnnotationDraft, Prediction, Task
+from tasks.openapi_schema import (
+    annotation_request_schema,
+    annotation_response_example,
+    dm_task_response_example,
+    prediction_request_schema,
+    prediction_response_example,
+    task_request_schema,
+    task_response_example,
+)
 from tasks.serializers import (
     AnnotationDraftSerializer,
     AnnotationSerializer,
@@ -47,15 +56,30 @@ logger = logging.getLogger(__name__)
     name='post',
     decorator=swagger_auto_schema(
         tags=['Tasks'],
+        x_fern_sdk_group_name='tasks',
+        x_fern_sdk_method_name='create',
+        x_fern_audiences=['public'],
         operation_summary='Create task',
         operation_description='Create a new labeling task in Label Studio.',
-        request_body=TaskSerializer,
+        request_body=task_request_schema,
+        responses={
+            '201': openapi.Response(
+                description='Created task', schema=TaskSerializer, examples={'application/json': task_response_example}
+            )
+        },
     ),
 )
 @method_decorator(
     name='get',
     decorator=swagger_auto_schema(
         tags=['Tasks'],
+        x_fern_sdk_group_name='tasks',
+        x_fern_sdk_method_name='list',
+        x_fern_pagination={
+            'offset': '$request.page',
+            'results': '$response.tasks',
+        },
+        x_fern_audiences=['public'],
         operation_summary='Get tasks list',
         operation_description="""
     Retrieve a list of tasks with pagination for a specific view or project, by using filters and ordering.
@@ -71,7 +95,72 @@ logger = logging.getLogger(__name__)
                 in_=openapi.IN_QUERY,
                 description='Resolve task data URIs using Cloud Storage',
             ),
+            openapi.Parameter(
+                name='fields',
+                type=openapi.TYPE_STRING,
+                enum=['all', 'task_only'],
+                default='task_only',
+                in_=openapi.IN_QUERY,
+                description='Set to "all" if you want to include annotations and predictions in the response',
+            ),
+            openapi.Parameter(
+                name='review',
+                type=openapi.TYPE_BOOLEAN,
+                in_=openapi.IN_QUERY,
+                description='Get tasks for review',
+            ),
+            openapi.Parameter(
+                name='include',
+                type=openapi.TYPE_STRING,
+                in_=openapi.IN_QUERY,
+                description='Specify which fields to include in the response',
+            ),
+            openapi.Parameter(
+                name='query',
+                type=openapi.TYPE_STRING,
+                in_=openapi.IN_QUERY,
+                description='Additional query to filter tasks. It must be JSON encoded string of dict containing '
+                'one of the following parameters: `{"filters": ..., "selectedItems": ..., "ordering": ...}`. Check '
+                '[Data Manager > Create View > see `data` field](#tag/Data-Manager/operation/api_dm_views_create) '
+                'for more details about filters, selectedItems and ordering.\n\n'
+                '* **filters**: dict with `"conjunction"` string (`"or"` or `"and"`) and list of filters in `"items"` array. '
+                'Each filter is a dictionary with keys: `"filter"`, `"operator"`, `"type"`, `"value"`. '
+                '[Read more about available filters](https://labelstud.io/sdk/data_manager.html)<br/>'
+                '                   Example: `{"conjunction": "or", "items": [{"filter": "filter:tasks:completed_at", "operator": "greater", "type": "Datetime", "value": "2021-01-01T00:00:00.000Z"}]}`\n'
+                '* **selectedItems**: dictionary with keys: `"all"`, `"included"`, `"excluded"`. If "all" is `false`, `"included"` must be used. If "all" is `true`, `"excluded"` must be used.<br/>'
+                '                   Examples: `{"all": false, "included": [1, 2, 3]}` or `{"all": true, "excluded": [4, 5]}`\n'
+                '* **ordering**: list of fields to order by. Currently, ordering is supported by only one parameter. <br/>\n'
+                '                   Example: `["completed_at"]`',
+            ),
         ],
+        responses={
+            '200': openapi.Response(
+                description='Tasks list',
+                schema=openapi.Schema(
+                    type=openapi.TYPE_OBJECT,
+                    properties={
+                        'tasks': openapi.Schema(
+                            description='List of tasks',
+                            type=openapi.TYPE_ARRAY,
+                            items=openapi.Schema(
+                                description='Task object',
+                                type=openapi.TYPE_OBJECT,
+                                # TODO: provide schema for DataManagerTaskSerializer
+                                # Right now the schema is defined in override.yml to ensure each item in paginated response is Task object derived from "#/components/schemas/Task"
+                                # We need to figure out more elegant way to define schema for DataManagerTaskSerializer to keep it in sync with Task object
+                            ),
+                        ),
+                        'total': openapi.Schema(description='Total number of tasks', type=openapi.TYPE_INTEGER),
+                        'total_annotations': openapi.Schema(
+                            description='Total number of annotations', type=openapi.TYPE_INTEGER
+                        ),
+                        'total_predictions': openapi.Schema(
+                            description='Total number of predictions', type=openapi.TYPE_INTEGER
+                        ),
+                    },
+                ),
+            )
+        },
     ),
 )
 class TaskListAPI(DMTaskListAPI):
@@ -107,6 +196,9 @@ class TaskListAPI(DMTaskListAPI):
     name='get',
     decorator=swagger_auto_schema(
         tags=['Tasks'],
+        x_fern_sdk_group_name='tasks',
+        x_fern_sdk_method_name='get',
+        x_fern_audiences=['public'],
         operation_summary='Get task',
         operation_description="""
         Get task data, metadata, annotations and other attributes for a specific labeling task by task ID.
@@ -114,29 +206,49 @@ class TaskListAPI(DMTaskListAPI):
         manual_parameters=[
             openapi.Parameter(name='id', type=openapi.TYPE_STRING, in_=openapi.IN_PATH, description='Task ID'),
         ],
+        request_body=no_body,
+        responses={
+            '200': openapi.Response(
+                description='Task',
+                schema=DataManagerTaskSerializer,
+                examples={'application/json': dm_task_response_example},
+            )
+        },
     ),
 )
 @method_decorator(
     name='patch',
     decorator=swagger_auto_schema(
         tags=['Tasks'],
+        x_fern_sdk_group_name='tasks',
+        x_fern_sdk_method_name='update',
+        x_fern_audiences=['public'],
         operation_summary='Update task',
         operation_description='Update the attributes of an existing labeling task.',
         manual_parameters=[
             openapi.Parameter(name='id', type=openapi.TYPE_STRING, in_=openapi.IN_PATH, description='Task ID'),
         ],
-        request_body=TaskSimpleSerializer,
+        request_body=task_request_schema,
+        responses={
+            '200': openapi.Response(
+                description='Updated task', schema=TaskSerializer, examples={'application/json': task_response_example}
+            )
+        },
     ),
 )
 @method_decorator(
     name='delete',
     decorator=swagger_auto_schema(
         tags=['Tasks'],
+        x_fern_sdk_group_name='tasks',
+        x_fern_sdk_method_name='delete',
+        x_fern_audiences=['public'],
         operation_summary='Delete task',
         operation_description='Delete a task in Label Studio. This action cannot be undone!',
         manual_parameters=[
             openapi.Parameter(name='id', type=openapi.TYPE_STRING, in_=openapi.IN_PATH, description='Task ID'),
         ],
+        request_body=no_body,
     ),
 )
 class TaskAPI(generics.RetrieveUpdateDestroyAPIView):
@@ -223,21 +335,6 @@ class TaskAPI(generics.RetrieveUpdateDestroyAPIView):
         else:
             return TaskSimpleSerializer
 
-    def retrieve(self, request, *args, **kwargs):
-        task = self.get_object()
-        project = task.project
-
-        # call machine learning api and format response
-        if project.evaluate_predictions_automatically:
-            for ml_backend in task.project.ml_backends.all():
-                ml_backend.predict_tasks([task])
-
-        result = self.get_serializer(task).data
-
-        # use proxy inlining to task data (for credential access)
-        result['data'] = task.resolve_uri(result['data'], project)
-        return Response(result)
-
     def patch(self, request, *args, **kwargs):
         return super(TaskAPI, self).patch(request, *args, **kwargs)
 
@@ -256,23 +353,48 @@ class TaskAPI(generics.RetrieveUpdateDestroyAPIView):
         tags=['Annotations'],
         operation_summary='Get annotation by its ID',
         operation_description='Retrieve a specific annotation for a task using the annotation result ID.',
+        x_fern_sdk_group_name='annotations',
+        x_fern_sdk_method_name='get',
+        x_fern_audiences=['public'],
+        request_body=no_body,
+        responses={
+            '200': openapi.Response(
+                description='Retrieved annotation',
+                schema=AnnotationSerializer,
+                examples={'application/json': annotation_response_example},
+            )
+        },
     ),
 )
 @method_decorator(
     name='patch',
     decorator=swagger_auto_schema(
         tags=['Annotations'],
+        x_fern_sdk_group_name='annotations',
+        x_fern_sdk_method_name='update',
+        x_fern_audiences=['public'],
         operation_summary='Update annotation',
         operation_description='Update existing attributes on an annotation.',
-        request_body=AnnotationSerializer,
+        request_body=annotation_request_schema,
+        responses={
+            '200': openapi.Response(
+                description='Updated annotation',
+                schema=AnnotationSerializer,
+                examples={'application/json': annotation_response_example},
+            )
+        },
     ),
 )
 @method_decorator(
     name='delete',
     decorator=swagger_auto_schema(
         tags=['Annotations'],
+        x_fern_sdk_group_name='annotations',
+        x_fern_sdk_method_name='delete',
+        x_fern_audiences=['public'],
         operation_summary='Delete annotation',
         operation_description="Delete an annotation. This action can't be undone!",
+        request_body=no_body,
     ),
 )
 class AnnotationAPI(generics.RetrieveUpdateDestroyAPIView):
@@ -329,17 +451,31 @@ class AnnotationAPI(generics.RetrieveUpdateDestroyAPIView):
     name='get',
     decorator=swagger_auto_schema(
         tags=['Annotations'],
+        x_fern_sdk_group_name='annotations',
+        x_fern_sdk_method_name='list',
+        x_fern_audiences=['public'],
         operation_summary='Get all task annotations',
         operation_description='List all annotations for a task.',
         manual_parameters=[
             openapi.Parameter(name='id', type=openapi.TYPE_INTEGER, in_=openapi.IN_PATH, description='Task ID'),
         ],
+        request_body=no_body,
+        responses={
+            '200': openapi.Response(
+                description='Annotation',
+                schema=AnnotationSerializer(many=True),
+                examples={'application/json': [annotation_response_example]},
+            )
+        },
     ),
 )
 @method_decorator(
     name='post',
     decorator=swagger_auto_schema(
         tags=['Annotations'],
+        x_fern_sdk_group_name='annotations',
+        x_fern_sdk_method_name='create',
+        x_fern_audiences=['public'],
         operation_summary='Create annotation',
         operation_description="""
         Add annotations to a task like an annotator does. The content of the result field depends on your 
@@ -360,7 +496,14 @@ class AnnotationAPI(generics.RetrieveUpdateDestroyAPIView):
         manual_parameters=[
             openapi.Parameter(name='id', type=openapi.TYPE_INTEGER, in_=openapi.IN_PATH, description='Task ID'),
         ],
-        request_body=AnnotationSerializer,
+        request_body=annotation_request_schema,
+        responses={
+            '201': openapi.Response(
+                description='Created annotation',
+                schema=AnnotationSerializer,
+                examples={'application/json': annotation_response_example},
+            )
+        },
     ),
 )
 class AnnotationsListAPI(GetParentObjectMixin, generics.ListCreateAPIView):
@@ -462,7 +605,6 @@ class AnnotationsListAPI(GetParentObjectMixin, generics.ListCreateAPIView):
 
 
 class AnnotationDraftListAPI(generics.ListCreateAPIView):
-
     parser_classes = (JSONParser, MultiPartParser, FormParser)
     serializer_class = AnnotationDraftSerializer
     permission_required = ViewClassPermission(
@@ -485,7 +627,6 @@ class AnnotationDraftListAPI(generics.ListCreateAPIView):
 
 
 class AnnotationDraftAPI(generics.RetrieveUpdateDestroyAPIView):
-
     parser_classes = (JSONParser, MultiPartParser, FormParser)
     serializer_class = AnnotationDraftSerializer
     queryset = AnnotationDraft.objects.all()
@@ -502,28 +643,75 @@ class AnnotationDraftAPI(generics.RetrieveUpdateDestroyAPIView):
     name='list',
     decorator=swagger_auto_schema(
         tags=['Predictions'],
+        x_fern_sdk_group_name='predictions',
+        x_fern_sdk_method_name='list',
+        x_fern_audiences=['public'],
         operation_summary='List predictions',
         filter_inspectors=[DjangoFilterDescriptionInspector],
         operation_description='List all predictions and their IDs.',
+        manual_parameters=[
+            openapi.Parameter(
+                name='task',
+                type=openapi.TYPE_INTEGER,
+                in_=openapi.IN_QUERY,
+                description='Filter predictions by task ID',
+            ),
+            openapi.Parameter(
+                name='project',
+                type=openapi.TYPE_INTEGER,
+                in_=openapi.IN_QUERY,
+                description='Filter predictions by project ID',
+            ),
+        ],
+        request_body=no_body,
+        responses={
+            '200': openapi.Response(
+                description='Predictions list',
+                schema=PredictionSerializer(many=True),
+                examples={'application/json': [prediction_response_example]},
+            )
+        },
     ),
 )
 @method_decorator(
     name='create',
     decorator=swagger_auto_schema(
         tags=['Predictions'],
+        x_fern_sdk_group_name='predictions',
+        x_fern_sdk_method_name='create',
+        x_fern_audiences=['public'],
         operation_summary='Create prediction',
         operation_description='Create a prediction for a specific task.',
+        request_body=prediction_request_schema,
+        responses={
+            '201': openapi.Response(
+                description='Created prediction',
+                schema=PredictionSerializer,
+                examples={'application/json': prediction_response_example},
+            )
+        },
     ),
 )
 @method_decorator(
     name='retrieve',
     decorator=swagger_auto_schema(
         tags=['Predictions'],
+        x_fern_sdk_group_name='predictions',
+        x_fern_sdk_method_name='get',
+        x_fern_audiences=['public'],
         operation_summary='Get prediction details',
         operation_description='Get details about a specific prediction by its ID.',
         manual_parameters=[
             openapi.Parameter(name='id', type=openapi.TYPE_INTEGER, in_=openapi.IN_PATH, description='Prediction ID'),
         ],
+        request_body=no_body,
+        responses={
+            '200': openapi.Response(
+                description='Prediction details',
+                schema=PredictionSerializer,
+                examples={'application/json': prediction_response_example},
+            )
+        },
     ),
 )
 @method_decorator(
@@ -531,32 +719,56 @@ class AnnotationDraftAPI(generics.RetrieveUpdateDestroyAPIView):
     decorator=swagger_auto_schema(
         tags=['Predictions'],
         operation_summary='Put prediction',
+        x_fern_audiences=['internal'],
         operation_description='Overwrite prediction data by prediction ID.',
         manual_parameters=[
             openapi.Parameter(name='id', type=openapi.TYPE_INTEGER, in_=openapi.IN_PATH, description='Prediction ID'),
         ],
+        request_body=prediction_request_schema,
+        responses={
+            '200': openapi.Response(
+                description='Updated prediction',
+                schema=PredictionSerializer,
+                examples={'application/json': prediction_response_example},
+            )
+        },
     ),
 )
 @method_decorator(
     name='partial_update',
     decorator=swagger_auto_schema(
         tags=['Predictions'],
+        x_fern_sdk_group_name='predictions',
+        x_fern_sdk_method_name='update',
+        x_fern_audiences=['public'],
         operation_summary='Update prediction',
         operation_description='Update prediction data by prediction ID.',
         manual_parameters=[
             openapi.Parameter(name='id', type=openapi.TYPE_INTEGER, in_=openapi.IN_PATH, description='Prediction ID'),
         ],
+        request_body=prediction_request_schema,
+        responses={
+            '200': openapi.Response(
+                description='Updated prediction',
+                schema=PredictionSerializer,
+                examples={'application/json': prediction_response_example},
+            )
+        },
     ),
 )
 @method_decorator(
     name='destroy',
     decorator=swagger_auto_schema(
         tags=['Predictions'],
+        x_fern_sdk_group_name='predictions',
+        x_fern_sdk_method_name='delete',
+        x_fern_audiences=['public'],
         operation_summary='Delete prediction',
         operation_description='Delete a prediction by prediction ID.',
         manual_parameters=[
             openapi.Parameter(name='id', type=openapi.TYPE_INTEGER, in_=openapi.IN_PATH, description='Prediction ID'),
         ],
+        request_body=no_body,
     ),
 )
 class PredictionAPI(viewsets.ModelViewSet):
@@ -580,6 +792,7 @@ class PredictionAPI(viewsets.ModelViewSet):
     name='post',
     decorator=swagger_auto_schema(
         tags=['Annotations'],
+        x_fern_audiences=['internal'],
         operation_summary='Convert annotation to draft',
         operation_description='Convert annotation to draft',
     ),
