@@ -14,6 +14,19 @@ class TaskMixin:
     def post_process_bulk_update_stats(cls, tasks) -> None:
         pass
 
+    def before_delete_actions(self):
+        """
+        Actions to execute before task deletion
+        """
+        pass
+
+    @staticmethod
+    def after_bulk_delete_actions(tasks_ids):
+        """
+        Actions to execute after bulk task deletion
+        """
+        pass
+
 
 class AnnotationMixin:
     def has_permission(self, user: 'User') -> bool:  # noqa: F821
