@@ -8,6 +8,7 @@ from core.label_config import replace_task_data_undefined_with_config_field
 from core.utils.common import load_func, retry_database_locked
 from django.conf import settings
 from django.db import IntegrityError, transaction
+from drf_yasg import openapi
 from projects.models import Project
 from rest_flex_fields import FlexFieldsModelSerializer
 from rest_framework import generics, serializers
@@ -29,8 +30,42 @@ class PredictionQuerySerializer(serializers.Serializer):
     project = serializers.IntegerField(required=False, help_text='Project ID to filter predictions')
 
 
+class PredictionResultField(serializers.JSONField):
+    class Meta:
+        swagger_schema_fields = {
+            'type': openapi.TYPE_ARRAY,
+            'title': 'Prediction result list',
+            'description': 'List of prediction results for the task',
+            'items': {
+                'type': openapi.TYPE_OBJECT,
+                'title': 'Prediction result items (regions)',
+                'description': 'List of predicted regions for the task',
+            },
+        }
+
+
+class AnnotationResultField(serializers.JSONField):
+    class Meta:
+        swagger_schema_fields = {
+            'type': openapi.TYPE_ARRAY,
+            'title': 'Annotation result list',
+            'description': 'List of annotation results for the task',
+            'items': {
+                'type': openapi.TYPE_OBJECT,
+                'title': 'Annotation result items (regions)',
+                'description': 'List of annotated regions for the task',
+            },
+        }
+
+
 class PredictionSerializer(ModelSerializer):
-    model_version = serializers.CharField(allow_blank=True, required=False)
+    result = PredictionResultField()
+    model_version = serializers.CharField(
+        allow_blank=True,
+        required=False,
+        help_text='Model version - tag for predictions that can be used to filter tasks in Data Manager, as well as '
+        'select specific model version for showing preannotations in the labeling interface',
+    )
     created_ago = serializers.CharField(default='', read_only=True, help_text='Delta time from creation time')
 
     class Meta:
@@ -51,6 +86,7 @@ class CompletedByDMSerializer(UserSerializer):
 class AnnotationSerializer(FlexFieldsModelSerializer):
     """ """
 
+    result = AnnotationResultField(required=False)
     created_username = serializers.SerializerMethodField(default='', read_only=True, help_text='Username string')
     created_ago = serializers.CharField(default='', read_only=True, help_text='Time delta from creation time')
     completed_by = serializers.PrimaryKeyRelatedField(required=False, queryset=User.objects.all())
@@ -314,11 +350,7 @@ class BaseTaskSerializerBulk(serializers.ListSerializer):
         ff_user = self.project.organization.created_by
 
         # get members from project, we need them to restore annotation.completed_by etc
-        organization = (
-            user.active_organization
-            if not self.project.created_by.active_organization
-            else self.project.created_by.active_organization
-        )
+        organization = self.project.organization
         members_email_to_id = dict(organization.members.values_list('user__email', 'user__id'))
         members_ids = set(members_email_to_id.values())
         logger.debug(f'{len(members_email_to_id)} members found in organization {organization}')
@@ -576,60 +608,8 @@ class TaskWithAnnotationsSerializer(TaskSerializer):
         exclude = ()
 
 
-class TaskIDWithAnnotationsSerializer(TaskSerializer):
-    """ """
-
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        # TODO: this called twice due to base class initializer
-        self.fields['annotations'] = AnnotationSerializer(many=True, default=[], context=self.context)
-
-    class Meta:
-        model = Task
-        fields = ['id', 'annotations']
-
-
-class TaskWithPredictionsSerializer(TaskSerializer):
-    """ """
-
-    predictions = PredictionSerializer(many=True, default=[], read_only=True)
-
-    class Meta:
-        model = Task
-        fields = '__all__'
-
-
-class TaskWithAnnotationsAndPredictionsSerializer(TaskSerializer):
-    predictions = PredictionSerializer(many=True, default=[], read_only=True)
-    annotations = serializers.SerializerMethodField(default=[], read_only=True)
-
-    def get_annotations(self, task):
-        annotations = task.annotations
-
-        if 'request' in self.context:
-            user = self.context['request'].user
-            if user.is_annotator:
-                annotations = annotations.filter(completed_by=user)
-
-        return AnnotationSerializer(annotations, many=True, read_only=True, default=True, context=self.context).data
-
-    @staticmethod
-    def generate_prediction(task):
-        """Generate prediction for task and store it to Prediction model"""
-        prediction = task.predictions.filter(model_version=task.project.model_version)
-        if not prediction.exists():
-            task.project.create_prediction(task)
-
-    def to_representation(self, instance):
-        self.generate_prediction(instance)
-        return super().to_representation(instance)
-
-    class Meta:
-        model = Task
-        exclude = ()
-
-
 class AnnotationDraftSerializer(ModelSerializer):
+
     user = serializers.CharField(default=serializers.CurrentUserDefault())
     created_username = serializers.SerializerMethodField(default='', read_only=True, help_text='User name string')
     created_ago = serializers.CharField(default='', read_only=True, help_text='Delta time from creation time')
@@ -711,13 +691,8 @@ class NextTaskSerializer(TaskWithAnnotationsAndPredictionsAndDraftsSerializer):
             return lock.unique_id
 
     def get_predictions(self, task):
-        project = task.project
-        if not project.show_collab_predictions:
-            return []
-        else:
-            for ml_backend in project.ml_backends.all():
-                ml_backend.predict_tasks([task])
-            return super().get_predictions(task)
+        predictions = task.get_predictions_for_prelabeling()
+        return PredictionSerializer(predictions, many=True, read_only=True, default=[], context=self.context).data
 
     def get_annotations(self, task):
         result = []
